@@ -1,15 +1,15 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
+
+import warnings
+from distutils.version import LooseVersion
+
 import numpy as np
 import pandas as pd
-import warnings
 
-from .pycompat import basestring, suppress, dask_array_type, OrderedDict
-from . import dtypes
-from . import formatting
-from . import ops
-from .utils import SortedKeysDict, not_implemented, Frozen
+from . import duck_array_ops, dtypes, formatting, ops
+from .arithmetic import SupportsArithmetic
+from .pycompat import OrderedDict, basestring, dask_array_type, suppress
+from .utils import Frozen, SortedKeysDict
 
 
 class ImplementsArrayReduce(object):
@@ -214,7 +214,7 @@ class AttrAccessMixin(object):
 def get_squeeze_dims(xarray_obj, dim, axis=None):
     """Get a list of dimensions to squeeze out.
     """
-    if not dim is None and not axis is None:
+    if dim is not None and axis is not None:
         raise ValueError('cannot use both parameters `axis` and `dim`')
 
     if dim is None and axis is None:
@@ -237,7 +237,7 @@ def get_squeeze_dims(xarray_obj, dim, axis=None):
     return dim
 
 
-class BaseDataObject(AttrAccessMixin):
+class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
     """Shared base class for Dataset and DataArray."""
 
     def squeeze(self, dim=None, drop=False, axis=None):
@@ -426,6 +426,11 @@ class BaseDataObject(AttrAccessMixin):
         grouped : GroupBy
             A `GroupBy` object patterned after `pandas.GroupBy` that can be
             iterated over in the form of `(unique_value, grouped_array)` pairs.
+
+        See Also
+        --------
+        core.groupby.DataArrayGroupBy
+        core.groupby.DatasetGroupBy
         """
         return self._groupby_cls(self, group, squeeze=squeeze)
 
@@ -485,9 +490,6 @@ class BaseDataObject(AttrAccessMixin):
         """
         Rolling window object.
 
-        Rolling window aggregations are much faster when bottleneck is
-        installed.
-
         Parameters
         ----------
         min_periods : int, default None
@@ -505,7 +507,8 @@ class BaseDataObject(AttrAccessMixin):
 
         Returns
         -------
-        rolling : type of input argument
+        Rolling object (core.rolling.DataArrayRolling for DataArray,
+        core.rolling.DatasetRolling for Dataset.)
 
         Examples
         --------
@@ -533,6 +536,11 @@ class BaseDataObject(AttrAccessMixin):
         array([  1.,   2.,   3.,   4.,   5.,   6.,   7.,   8.,   9.,  10.])
         Coordinates:
           * time     (time) datetime64[ns] 2000-02-15 2000-03-15 2000-04-15 ...
+
+        See Also
+        --------
+        core.rolling.DataArrayRolling
+        core.rolling.DatasetRolling
         """
 
         return self._rolling_cls(self, min_periods=min_periods,
@@ -737,18 +745,61 @@ class BaseDataObject(AttrAccessMixin):
             self._file_obj.close()
         self._file_obj = None
 
+    def isin(self, test_elements):
+        """Tests each value in the array for whether it is in the supplied list.
+
+        Parameters
+        ----------
+        test_elements : array_like
+            The values against which to test each value of `element`.
+            This argument is flattened if an array or array_like.
+            See numpy notes for behavior with non-array-like parameters.
+
+        Returns
+        -------
+        isin : same as object, bool
+            Has the same shape as this object.
+
+        Examples
+        --------
+
+        >>> array = xr.DataArray([1, 2, 3], dims='x')
+        >>> array.isin([1, 3])
+        <xarray.DataArray (x: 3)>
+        array([ True, False,  True])
+        Dimensions without coordinates: x
+
+        See also
+        --------
+        numpy.isin
+        """
+        from .computation import apply_ufunc
+        from .dataset import Dataset
+        from .dataarray import DataArray
+        from .variable import Variable
+
+        if isinstance(test_elements, Dataset):
+            raise TypeError(
+                'isin() argument must be convertible to an array: {}'
+                .format(test_elements))
+        elif isinstance(test_elements, (Variable, DataArray)):
+            # need to explicitly pull out data to support dask arrays as the
+            # second argument
+            test_elements = test_elements.data
+
+        return apply_ufunc(
+            duck_array_ops.isin,
+            self,
+            kwargs=dict(test_elements=test_elements),
+            dask='allowed',
+        )
+
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
         self.close()
 
-    # this has no runtime function - these are listed so IDEs know these
-    # methods are defined and don't warn on these operations
-    __lt__ = __le__ = __ge__ = __gt__ = __add__ = __sub__ = __mul__ = \
-        __truediv__ = __floordiv__ = __mod__ = __pow__ = __and__ = __xor__ = \
-        __or__ = __div__ = __eq__ = __ne__ = not_implemented
-
 
 def full_like(other, fill_value, dtype=None):
     """Return a new object with the same shape and type as a given object.
